#include <stdlib.h>
#include "ctapi.h"
#include "ctbcs.h"

// liczba reprezentujca nasz terminal
#define CTERMINAL 1
// makro zwracajce okrelony bit (nbit) z liczby numb 
#define BINARY(numb, nbit) ((numb>>nbit & 1) == 0 ? '0' : '1')

// funkcja informujca o bdach
char error(char *fun, char rv)
{
    printf("%s (%i) ", fun, rv);
    switch (rv) {
    case OK:
	printf("OK (Success)\n");
	return OK;
    case ERR_INVALID:
	printf("ERR_INVALID (Invalid Data)\n");
	return ERR_INVALID;
    case ERR_CT:
	printf("ERR_CT (Cardterminal Error)\n");
	return ERR_CT;
    case ERR_TRANS:
	printf("ERR_TRANS (Transmission Error)\n");
	return ERR_TRANS;
    case ERR_MEMORY:
	printf("ERR_MEMORY (Memory Allocation Error)\n");
	return ERR_MEMORY;
    case ERR_HTSI:
	printf("ERR_TRANS (Host Transport Service Interface Error)\n");
	return ERR_HTSI;
    }
    return OK;
}

// funkcja pobiera ATR z terminala o numerze ctn 
char get_atr(unsigned short ctn)
{
    char rv;
    unsigned char dad, sad, command[16], response[256];
    unsigned short lenr;
    unsigned short lenc;
    unsigned short p;

    // sprawdzamy czy karta zostaa umieszczona w interfejsie 
    command[0] = CTBCS_CLA;
    command[1] = CTBCS_INS_STATUS;
    command[2] = CTBCS_P1_CT_KERNEL;
    command[3] = CTBCS_P2_STATUS_ICC;
    command[4] = 0x00;

    dad = 1;
    sad = 2;
    lenr = 256;

    // wysanie komendy 
    rv = CT_data(ctn, &dad, &sad, 11, command, &lenr, response);

    // wystpi bd 
    if ((rv != OK) || (response[lenr - 2] != 0x90)) {
	error("CT_data (terminal status)", rv);
	return 0;
    }

    // karta jest zaczona 
    if (response[0] == CTBCS_DATA_STATUS_CARD_CONNECT) {
	printf("Activiting card...\n");

	// uruchamiamy kart 
	command[0] = CTBCS_CLA;
	command[1] = CTBCS_INS_REQUEST;
	command[2] = CTBCS_P1_INTERFACE1;
	command[3] = CTBCS_P2_REQUEST_GET_ATR;
	command[4] = 0x00;

	dad = 1;
	sad = 2;
	lenr = 256;

	rv = CT_data(ctn, &dad, &sad, 5, command, &lenr, response);

	if ((rv != OK) || (response[lenr - 2] != 0x90)) {
	    error("CT_data (activiting card)", rv);
	    return 0;
	}

	// wydruk ATR 
	for (p = 0; p < lenr - 2; ++p) {
	    unsigned short bin;
	    printf("[%s%d]  0x%s%X ", p < 10 ? "0" : "", p,
		   response[p] < 16 ? "0" : "", response[p]);
	    for (bin = 8; bin > 0; --bin)
		printf("%c", BINARY(response[p], bin - 1));
	    printf(" [%c]\n", response[p]);

	}
	return 1;
    } else {
	printf("Please insert a smartcard in the terminal\n");
    }

    return 0;
}

int main(int argc, char *argv[])
{
    char rv;

    // incjowanie poczenia do termnala podczonego do COM1 
    rv = CT_init(CTERMINAL, PORT_COM1);
    if (error("CT_init", rv) != OK)
	return 1;

    get_atr(CTERMINAL);

    // zamykanie poczenia zwizanego z naszym poczeniem 
    rv = CT_close(CTERMINAL);
    if (error("CT_close", rv) != OK)
	return 1;

    return 0;
}


