// import niezbdnych klas
import java.util.Enumeration;
import java.util.Properties;

import opencard.core.service.SmartCard;
import opencard.core.service.CardServiceException;

import opencard.core.terminal.CardTerminalRegistry;
import opencard.core.terminal.CardTerminal;
import opencard.core.terminal.SlotChannel;
import opencard.core.terminal.CardID;
import opencard.core.terminal.CommandAPDU;
import opencard.core.terminal.ResponseAPDU;
import opencard.core.terminal.CardTerminalException;

import opencard.core.util.HexString;
import opencard.core.util.OpenCardPropertyLoadingException;

public class OCFExample1
{
    public static void main (String[] args)
    {
	// rejestr czytnikw w systemie
	CardTerminalRegistry ctr;

	// lista terminali
	Enumeration terminals;
	
	// terminal (czytnik)
	CardTerminal terminal = null;
	
	// aktywny slot terminala (czytnika)
	SlotChannel sch;
	
	// komenda "get challenge" (CLA INS P1 P2 LE)
	final byte[] GET_CHALLENGE={(byte) 0x00, (byte) 0x84, 
				    (byte) 0x00, (byte) 0x00, 
				    (byte) 0x08};

	try
	{
	    // prba uruchomienia usug zwizanych z kartami inteligentnymi
	    SmartCard.start();

	    // pobranie rejestru terminali
	    ctr = CardTerminalRegistry.getRegistry();
	    
	    // pobranie listy terminali z rejestru
	    terminals = ctr.getCardTerminals();

	    // dla wszystkich terminali z rejestru
	    while (terminals.hasMoreElements()) 
	    {
		terminal = (CardTerminal) terminals.nextElement();

		// dane o terminalu 
		System.out.println("Address : "+terminal.getAddress());
		System.out.println("Name    : "+terminal.getName());
		System.out.println("Type    : "+terminal.getType());

		// pobranie iloci slotw w terminalu
		int slots = terminal.getSlots();

		System.out.println("Slots number : "+slots);

		// wydruk waciwoci terminala
		Properties termProps = terminal.features ();
		if (termProps!=null) 
		{
		    System.out.println("Terminal properties : ");
		    termProps.list(System.out);
		}
		else
		    System.out.println("No terminal properties.");
		

		// sprawdzenie kadego slotu terminala
	        for (int slotID=0; slotID<slots; slotID++) 
		{
		    // sprawdzenie dostpnoci slotu
		    System.out.println("Slot " + slotID +
			    " channel available : " + 
			    (terminal.isSlotChannelAvailable(slotID) ? 
			    "yes" : "no"));
		    try 
		    {
			// otwarcie slotu
			sch=terminal.openSlotChannel(slotID);

			// sprawdzenie czy w slocie znajduje si karta
			System.out.println("Card present : " + 
				(terminal.isCardPresent(slotID) ? 
				 "yes" : "no") );
			
			// informacje o karcie 
			CardID cid = sch.getCardID();

			// wydruk ATR karty
			System.out.println ("ATR : " + 
				HexString.hexify(cid.getATR()));

			// utworzenie komendy GET CHALLENGE
			CommandAPDU  apdu = new CommandAPDU(GET_CHALLENGE);
			// przesanie komendy do karty
			ResponseAPDU rpdu = sch.sendAPDU(apdu);

			// wydruk otrzymanych wynikw
			System.out.println("Response data : " + 
				HexString.hexify(rpdu.data()));

			System.out.println("Response SW : " + 
				HexString.hexifyShort(rpdu.sw()));
		    }
		    catch(CardTerminalException ex)
		    {
			System.out.println("CardTerminalException : ");
			System.out.println(ex.getMessage());
		    }
		    System.out.println();
		}
	    }
	    // zamknicie usug zwizanych z kartami inteligentnymi
	    SmartCard.shutdown();
	}
	// wyjtek zwizany z usugami systemowymi
	catch (CardServiceException ex) 
	{
            System.out.println("CardServiceException : ");
            System.out.println(ex.getMessage());
        }
	// wyjtek zgaszany przy bdach terminala
	catch (CardTerminalException ex) 
	{
            System.out.println("CardTerminalException : ");
            System.out.println(ex.getMessage());
        }
	// wyjtek zgaszany w sytuacji nieprawidowego 
	// pliku konfiguracyjnego lub jego braku
	catch (OpenCardPropertyLoadingException ex) 
	{
            System.out.println("OpenCardPropertyLoadingException : ");
            System.out.println(ex.getMessage());
        }
	// w przypadku braku dostpu do odpowiednich klas
	catch (ClassNotFoundException ex) 
	{
            System.out.println("ClassNotFoundException : ");
            System.out.println(ex.getMessage());
        } 
    }
}

