// doczenie niezbdnych plikw nagwkowych
#include <stdio.h>
#include <stdlib.h>

// prototypy funkcji OpenCT
#include <openct.h>

int main(int argc, char **argv)
{
    // komenda GET CHALLENGE
    unsigned char GET_CHALLENGE[] = {0x00, 0x84, 0x00, 0x00, 0x08};

    // bufory na ATR i odpowied od karty
    unsigned char atr[33];
    unsigned char res[10];
    
    // kontekst poczenia
    ct_handle       *h;
    // blokada
    ct_lock_handle  lock;
    
    // pomocnicze zmienne
    int reader, i, rsp;

    // wydruk czytnikw dostpnych w systemie
    for (i=0; i<OPENCT_MAX_READERS; i++) 
    {
	ct_info_t info;
	// pobranie informacji o czytniku 
	if (ct_reader_info(i, &info) < 0)
	    continue;
	printf("Reader %02d: ", i);
	// wydruk nazwy czytnika
	printf("%s\n", info.ct_name);
    }
    
    // wybr czytnika do dalszych operacji
    do
    {
	printf("Select reader : ");
	scanf("%d", &reader);
    }
    while (reader>OPENCT_MAX_READERS || reader<=0);

    // podczenie do czytnika
    if (!(h = ct_reader_connect(reader))) 
    {
	printf("Failed to connect %02d\n", reader);
	return 1;
    }
    
    // naoenie blokady
    if ((rsp=ct_card_lock(h, 0, IFD_LOCK_EXCLUSIVE, &lock))<0) 
    {
        printf("Fatled to lock the card [%d]\n", rsp);
	return 1;
    }

    // zerowanie karty
    rsp=do_reset(h, atr, sizeof(atr));
	    
    // wydruk ATR
    printf("ATR [%d] = ", rsp);
    for (i=0; i<rsp; +i)
    {
        printf("%02X ", atr[i]);
    }
    printf("\n");

    // przesanie komedy do karty
    rsp=ct_card_transact(h, 0, GET_CHALLENGE, sizeof(GET_CHALLENGE), 
			 res, sizeof(res));
    if (rsp<0) 
    {
        printf("Fatled to send GET_CHALLENGE [%d]\n", rsp);
	return 1;
    }
	
    // wydruk odpowiedzi
    printf("RAPDU [%d] = ", rsp);
    for (i=0; i<rsp; +i)
    {
        printf("%02X ", res[i]);
    }
    printf("\n");

    // zdjcie blokady
    ct_card_unlock(h, 0, lock);
    if (rsp<0) 
    {
        printf("Failed to unlock [%d]\n", rsp);
	return 1;
    }

    // zamknicie poczenia
    ct_reader_disconnect(h);

    return 0;
}
