#!/usr/bin/perl

# doczenie niezbdnych pakietw
use ExtUtils::testlib;
use Chipcard::PCSC;
use Chipcard::PCSC::Card;

use warnings;
use strict;

# poczenie do zarzdcy zasobw
my $hContext;
# poczenie do czytnika
my $hCard;
# lista czytnikw
my @readers;
# status
my @status;

# pomocnicze zmienne
my $resp;
my $sw;
my $iReader;
my $rv;

# komenda GET CHALLENGE
my $GET_CHALLENGE = Chipcard::PCSC::ascii_to_array ("00 84 00 00 08");

# poczenie do zarzdcy zasobw
print "Connecting PC/SC manager : ";
$hContext = new Chipcard::PCSC();
die ("failed ($Chipcard::PCSC::errno)\n") 
    unless (defined $hContext);
print "success\n";

# odczytanie listy czytnikw w systemie
print "Readers list : ";
@readers = $hContext->ListReaders ();
die ("failed ($Chipcard::PCSC::errno)\n") 
    unless (defined($readers[0]));
print "success\n";

# wydruk listy dostpnych czytnikw
for ($iReader=0; $iReader<=$#readers; ++$iReader)
{
    print "[".($iReader+1)."] ";
    print $readers[$iReader]."\n";
}

# wybr czytnika
do
{
    print "Select reader : ";
    $iReader = <STDIN>;
    $iReader--;
}
while ($iReader<0 || $iReader>$#readers);

# nawizanie poczenia
print "Connecting to the reader : ";
$hCard = new Chipcard::PCSC::Card ($hContext);
die ("failed ($Chipcard::PCSC::errno)\n") 
    unless (defined($hCard));
print "success\n";

# poczenie do czytnika i karty
print "Connecting to the card : ";
$rv = $hCard->Connect($readers[$iReader], $Chipcard::PCSC::SCARD_SHARE_SHARED);
unless ($rv) 
{
    print "\nReconnect : ";
    $rv = $hCard->Reconnect ($Chipcard::PCSC::SCARD_SHARE_SHARED, 
				 $Chipcard::PCSC::SCARD_PROTOCOL_T0 , 
				 $Chipcard::PCSC::SCARD_RESET_CARD);
    die ("failed ($Chipcard::PCSC::errno)\n") 
	unless ($rv);    
    print "success\n";
}
die ("Unknown protocol : $hCard->{dwProtocol}\n")
    unless ($hCard->{dwProtocol}==$Chipcard::PCSC::SCARD_PROTOCOL_T0 ||
            $hCard->{dwProtocol}==$Chipcard::PCSC::SCARD_PROTOCOL_T1 ||
	    $hCard->{dwProtocol}==$Chipcard::PCSC::SCARD_PROTOCOL_RAW);
print "success\n";

# ustawienie czasu oczekiwania
print ("Setting up timeout value : ");
die ("failed ($Chipcard::PCSC::errno)\n") 
    unless ($hContext->SetTimeout (50));
print "success\n";

# pobranie statusu poczenia
print "Getting status : ";
@status = $hCard->Status();
die ("failed ($Chipcard::PCSC::errno)\n") 
    unless ($status[0]);
print "success\n";

print "Reader name : $status[0]\n";
print "State : $status[1]\n";
print "Current protocol : $status[2]\n";
print "ATR : " . Chipcard::PCSC::array_to_ascii ($status[3]) . "\n";

# rozpoczcie transakcji
print ("Transaction begin : ");
die ("failed ($Chipcard::PCSC::errno)\n") 
    unless ($hCard->BeginTransaction());
print "success\n";

# wysanie komendy
print ("Exchanging data : ");
$resp = $hCard->Transmit($GET_CHALLENGE);
die ("failed ($Chipcard::PCSC::errno)\n") 
    unless (defined ($resp));
print "success\n";

print "Card response : " . Chipcard::PCSC::array_to_ascii ($resp) . "\n";

# wysanie komendy
$GET_CHALLENGE = "00 84 00 00 08";
print "TransmitWithCheck : \n";
# akceptacja dowolnej (".. ..") odpowiedzi karty zawartej w SW
($sw, $resp) = $hCard->TransmitWithCheck($GET_CHALLENGE, ".. ..", 1);
warn "TransmitWithCheck: $Chipcard::PCSC::Card::Error" 
    unless defined $sw;

print "Card response : $resp (SW: $sw)\n";

# wydruk informacji zawartej w SW
print "ISO7816Error : ";
print "$sw  (" . &Chipcard::PCSC::Card::ISO7816Error($sw) . ")\n";

# zakoczenie transakcji
print ("Transaction end : ");
die ("failed ($Chipcard::PCSC::errno)\n") 
    unless ($hCard->EndTransaction($Chipcard::PCSC::SCARD_LEAVE_CARD));
print "success\n";

# zakoczenie poczenia z kart
print "Disconnecting the card : ";
$rv = $hCard->Disconnect($Chipcard::PCSC::SCARD_LEAVE_CARD);
die ("failed ($Chipcard::PCSC::errno)\n") 
    unless $rv;
print "success\n";

# zakoczenie poczenia z czytnikiem
print "Disconnecting reader : ";
$hCard = undef;
print "success\n";

# zakoczenie poczenia z zarzdc zasobw
print "Disconnect PC/SC manager : ";
$hContext = undef;
print "success\n";
