#include <stdio.h>
#include <assert.h>
#include <string.h>

// definicje zalene od platformy systemowej
#ifndef _WIN32

#define CK_PTR *

#define CK_DEFINE_FUNCTION(returnType, name) \
   returnType name

#define CK_DECLARE_FUNCTION(returnType, name) \
   returnType name

#define CK_DECLARE_FUNCTION_POINTER(returnType, name) \
   returnType (* name)

#define CK_CALLBACK_FUNCTION(returnType, name) \
   returnType (* name)

#ifndef NULL_PTR
#define NULL_PTR 0
#endif

#else

#define CK_PTR *

#define CK_DEFINE_FUNCTION(returnType, name) \
   returnType __declspec(dllexport) name

#define CK_DECLARE_FUNCTION(returnType, name) \
   returnType __declspec(dllexport) name

#define CK_DECLARE_FUNCTION_POINTER(returnType, name) \
   returnType __declspec(dllimport) (* name)

#define CK_CALLBACK_FUNCTION(returnType, name) \
   returnType (* name)

#ifndef NULL_PTR
#define NULL_PTR 0
#endif

#endif

#include "pkcs11.h"

int main()
{
    // warto zwracana przez funkcje
    CK_RV rv;
    // sesja
    CK_SESSION_HANDLE session = CK_INVALID_HANDLE;
    // obiekt
    CK_OBJECT_HANDLE object = CK_INVALID_HANDLE;
    // slot 
    CK_SLOT_ID slot = 0;
    // flagi
    CK_FLAGS flags = CKF_RW_SESSION | CKF_SERIAL_SESSION;

    // uchwyt do biblioteki
    CK_FUNCTION_LIST_PTR p11;

    // adowanie biblioteki dynamicznej i pobranie uchwytu do listy funkcji
    // np. dla WIN32
    HINSTANCE hModule = LoadLibrary("pkcs11.dll");
    assert(hModule!=NULL);
    (*GetProcAddress(hModule, "C_GetFunctionList"))(&p11);
    
    // inicjalizacja moduu
    rv = p11->C_Initialize(NULL);
    if (rv != CKR_OK)
    {
	printf("C_Initialize error [%d]", rv);
	return 1;
    }

    // pobranie informacji o slotach w systemie i ustalenie wartoci
    // zmiennej slot
    // ...
    
    // otwarcie sesji
    rv = p11->C_OpenSession(slot, flags, NULL, NULL, &session);
    if (rv != CKR_OK)
    {
	printf("C_OpenSession error [%d]", rv);
	return 1;
    }

    // ...
    // operacje w czasie sesji 
    // ...

    // zamknicie sesji
    p11->C_CloseSession(session);
    
    // zakoczenie korzystania z moduu
    p11->C_Finalize(NULL_PTR);

    // wyadowanie biblioteki dynamicznej
    // ...

    return 0;
};
