// nazwa pakietu
package pl.edu.pw.ii.scproj.samples.PurseExample;

// import niezbdnych pakietw
import javacard.framework.*;

// klasa PurseExample realizujca funkcjonalno elektronicznej portmonetki
public class PurseExample extends Applet
{
    // stae wykorzystywane przy analizie komend APDU
    
    // klasa komend APDU elektronicznej portmonetki
    private final static byte CLA_PURSE       = (byte) 0x90;
    // pobranie informacji o zasobach w portfelu
    private final static byte CMD_GET_BALANCE = (byte) 0x50;
    // dodanie zasobw
    private final static byte CMD_CREDIT      = (byte) 0x52;
    // wydanie pienidzy
    private final static byte CMD_DEBIT       = (byte) 0x54;
    // weryfikacja kodu PIN
    private final static byte CMD_VERIFY_PIN  = (byte) 0x56;

    // kod odpowiedzi po niepoprawnym wprowadzeniu kodu PIN
    private final static short SW_WRONG_PIN = (short) 0x63C0;

    // maksymalna warto iloci pienidzy w portmonetce (300,00 PLN)
    private static final short maxBalance = 30000;

    // aktualna ilo pienidzy w portmonetce
    private short balance;

    // kod PIN 
    private OwnerPIN creditPIN;
   
    // stworzenie instancji apletu na karcie
    private PurseExample(byte[] buffer, short offset, byte length)
    {
        short paramOffset = offset;

        if(length > 9) 
	{
            // parametry instalacyjne przekazane zgodnie z OpenPlatform 2.0.1

	    // przesunicie do danych okrelajcych prawa apletu
            paramOffset += (short)(1 + buffer[offset]);
	    // przesunicie do parametrw aplikacji
            paramOffset += (short)(1 + buffer[paramOffset]);
	    // oczekiwany jest PIN o dugoci 4 bajtw
            if(buffer[paramOffset] != 4)
                // operacja koczy si z bdem
                ISOException.throwIt(
		    (short)(ISO7816.SW_WRONG_LENGTH+offset+length-paramOffset));
            // dane aplikacji (kod PIN)
            paramOffset++;
        } 
	else 
	{
            // parametry instalacyjne przekazane zgodnie z OpenPlatform 2.0.
            if(length != 4)
                ISOException.throwIt((short)(ISO7816.SW_WRONG_LENGTH + length));
        } 
	
	// stworzenie instancji klasy OwnerPIN: kod PIN o dugoci 4,
	// ilo prb wprowadzenia - 3
	creditPIN = new OwnerPIN ((byte)3, (byte)4);

	// ustalenie nowego kodu PIN (na podstawie danych podanych przy tworzeniu instancji
	creditPIN.update(buffer, paramOffset,(byte)4);

	// pocztkowa ilo pienidzy w portmonetce
        balance = 0;

	// rejestracja instancji apletu
	register(buffer, (short)(offset+1) ,(byte)(buffer[offset]));
    }

    // stworzenie instancji apletu
    public static void install(byte[] bArray, short bOffset, byte bLength) 
	throws ISOException
    {
	// wywoanie konstruktora
        new PurseExample (bArray, bOffset, (byte)bLength);
    }

    // wywoywane po wybraniu apletu
    public boolean select()
    {
        return true;
    }

    // wywoywane po zakoczeniu korzystania z apletu
    public void deselect()
    {
	// zerowanie statusu kodu PIN
	creditPIN.reset();
    }

    // przetwarzanie komend APDU 
    public void process(APDU apdu) throws ISOException
    {
        // pobranie bufora z APDU
        byte[] apduBuffer = apdu.getBuffer();
	// ilo danych w buforze
        byte dataLength = apduBuffer[ISO7816.OFFSET_LC];

	// przechwycienie komendy SELECT FILE
	if (selectingApplet()) 
            ISOException.throwIt(ISO7816.SW_NO_ERROR);

	// nie obsugujemy kanaw logicznych 
	apduBuffer[ISO7816.OFFSET_CLA] =
           (byte)(buffer[ISO7816.OFFSET_CLA] & (byte)0xFC);
	
        // sprawdzenie poprawnoci klasy instrukcji
        if (apduBuffer[ISO7816.OFFSET_CLA] != CLA_PURSE)
            ISOException.throwIt(ISO7816.SW_CLA_NOT_SUPPORTED);

	// w zalenoci od przesanej instrukcji podejmowane s rne operacje
	switch(apdu.getBuffer()[ISO7816.OFFSET_INS])
	{
	    // pobranie iloci gotwki w portfelu
	    case CMD_GET_BALANCE:
		getBalance(apdu);
		break;

	    // doadowanie
	    case CMD_CREDIT:
		creditPurse(apdu);
		break;

	    // transakcja
	    case CMD_DEBIT:
		debitPurse(apdu);
		break;

	    // weryfikacja kodu PIN
	    case CMD_VERIFY_PIN:
		verifyPIN(apdu);
		break;

	    // kada inna instrukcja jest niepoprawna
	    default:
		// zgoszenie wyjtku
		ISOException.throwIt(ISO7816.SW_INS_NOT_SUPPORTED);
		break;
	}
    }

    // zwraca aktualn ilo gotwki w portfelu
    private void getBalance(APDU apdu)
    {
        // pobranie bufora z APDU
        byte[] apduBuffer = apdu.getBuffer();

        // wpisanie aktualnej iloci gotwki (za komend APDU)
        apduBuffer[5] = (byte)(balance >> 8);
        apduBuffer[6] = (byte)balance;

	// przejcie w tryb odpowiedzi
        apdu.setOutgoing(); 
	// dugo odpowiedzi
        apdu.setOutgoingLength((short)2); 

        // przesanie odpowiedzi APDU (2 bajty od bajtu 5 z bufora)
        apdu.sendBytes((short)5, (short)2);
    }

    // doadowanie portmonetki
    private void creditPurse(APDU apdu) throws ISOException
    {
	// sprawdzenie czy wczeniej podano prawidowy kod PIN
        if (!creditPIN.isValidated())
            ISOException.throwIt(ISO7816.SW_SECURITY_STATUS_NOT_SATISFIED);

        // pobranie bufora z APDU
        byte[] apduBuffer = apdu.getBuffer();

        // sprawdzenie czy uytkownik dostarczy 2 bajty danych (kwota wypaty) 
	// i czy udao si je odebra
        if( apduBuffer[4] != 2 || apdu.setIncomingAndReceive() != 2 )
            ISOException.throwIt(ISO7816.SW_WRONG_LENGTH);

        // odczytanie wartoci doadowania (bajty 5 i 6 w przesaniej komendzie APDU)
        short amount = (short)(((apduBuffer[5] << 8 ) & (short)0xFF00) +
		       ((apduBuffer[6]) & (short)0x00FF));

        // spawdzenie poprawnoci wartoci doadowania
        if ((amount > (short)(maxBalance - balance)) || (amount <= (short)0))
	    ISOException.throwIt(ISO7816.SW_DATA_INVALID);
        else
	    // doadowanie eklektronicznej gotwki
            balance += amount;
    }

    // wypata gotwki
    private void debitPurse(APDU apdu) throws ISOException
    {
        // pobranie bufora z APDU
        byte[] apduBuffer = apdu.getBuffer();

        // sprawdzenie czy uytkownik dostarczy 2 bajty danych (kwota wypaty) 
	// i czy udao si je odebra
        if(apduBuffer[4] != 2 || apdu.setIncomingAndReceive() != 2)
        {
	    // operacja zakoczona z bdem
	    ISOException.throwIt(ISO7816.SW_WRONG_LENGTH) ;
        }

	// odczytanie wartoci transakcji (bajty 5 i 6 w przesaniej komendzie APDU)
        short amount = (short)(((apduBuffer[5] << 8 ) & (short)0xFF00) +
		       ((apduBuffer[6]) & (short)0x00FF));

        // sprawdzenie czy kwota jest poprawna
        if ((balance >= amount) && (amount > 0))
        {
	    // uaktualnienie stanu portfela
	    balance -= amount ;

	    // nowy stan portfela zostanie przesany w odpowiedzi
	    apduBuffer[9] = (byte)(balance >> 8);
	    apduBuffer[10] = (byte)balance;

	    // przesanie 2 bajtw odpowiedzi
	    apdu.setOutgoing(); 
	    apdu.setOutgoingLength((short)2);
	    apdu.sendBytes((short)9, (short)2);
        }
        else
	    // kwota nie jest poprawna
	    throw new ISOException (ISO7816.SW_DATA_INVALID) ;
    }

    // weryfikacja kodu PIN
    private void verifyPIN(APDU apdu)
    {
	// pobranie bufora z APDU
        byte[] buffer = apdu.getBuffer();
	
	// dugo kodu PIN
        byte pinLength = buffer[ISO7816.OFFSET_LC];

	// pobranie przekazanych danych
	if(apdu.setIncomingAndReceive()<pinLength) 
	    ISOException.throwIt(ISO7816.SW_WRONG_LENGTH);
        
	if(!creditPIN.check(buffer, ISO7816.OFFSET_CDATA, pinLength))
        {
	    // sprawdzenie kodu PIN nie powiodo si
            byte triesRemaining = creditPIN.getTriesRemaining();
	    // ilo pozostaych prb wynosi x (63 Cx)
            ISOException.throwIt((short)(SW_WRONG_PIN + triesRemaining));
        }
    }
}
